import logging
from pathlib import Path
from typing import Tuple

import typer
from packaging.version import parse
from typing_extensions import Annotated

from . import __version__
from .logger import logger
from .pycracks import run

app = typer.Typer(add_completion=False)


@app.callback(invoke_without_command=True)
def main(
    test_command: Annotated[
        str, typer.Option("--test-command", "-c", help="Command that runs the test")
    ] = "python -m pipenv run test",
    paths: Annotated[
        Tuple[str], typer.Option("--path", "-p", help="Paths to checkout")
    ] = ("tests",),
    target_version: Annotated[
        str, typer.Option("--target-version", "-t", help="Version to release")
    ] = "1.0.0",
    version: Annotated[
        bool, typer.Option("--version", "-V", help="Shows the version of pycracks")
    ] = False,
    verbose: Annotated[
        bool, typer.Option("--verbose", "-v", help="Print INFO logging statements")
    ] = False,
    extra_verbose: Annotated[
        bool, typer.Option("-vv", help="Print DEBUG logging statements")
    ] = False,
) -> None:
    if version:
        typer.echo(__version__)
        raise typer.Exit()

    if verbose:
        logging.basicConfig(level=logging.INFO)

    if extra_verbose:
        logging.basicConfig(level=logging.DEBUG)

    parsed_target_version = parse(target_version)
    paths_ = (Path(path) for path in paths)

    run_succeded = run(test_command, paths_, parsed_target_version)

    if not run_succeded:
        logger.error("Breaking Changes detected and major not updated")
        raise typer.Exit(code=1)

    logger.info("No incompatibilities between changes and version")
    raise typer.Exit()


if __name__ == "__main__":
    app()



import logging

logger = logging.getLogger(__name__)

handler = logging.StreamHandler()

format_string = (
    "%(asctime)s - %(levelname)s - %(filename)s:%(lineno)d - %(funcName)s - %(message)s"
)
formatter = logging.Formatter(format_string)
handler.setFormatter(formatter)

logger.addHandler(handler)



from __future__ import annotations

import subprocess
from pathlib import Path
from typing import Iterable

import git.exc
from git import Repo
from packaging.version import Version, parse

from .logger import logger


def run(test_command: str, paths: Iterable[Path], target_version: Version) -> bool:
    repo = get_repo()
    fetch_tags(repo)

    literal_latest_version = get_latest_version(repo)
    latest_version = parse(literal_latest_version)

    if target_version <= latest_version:
        logger.error("Target version should be greater than last version")
        return False

    chekcout_paths_from_reference(repo, literal_latest_version, paths)

    test_succeeded = run_test_command(test_command)

    discard_changes(repo)

    return is_breaking_change_expected(target_version, latest_version, test_succeeded)


def is_breaking_change_expected(
    target_version: Version, latest_version: Version, test_succeeded: bool
) -> bool:
    major_increased = target_version.major > latest_version.major
    return major_increased or test_succeeded


def get_repo(path: Path = Path(".")) -> Repo:
    try:
        return Repo(path)
    except git.exc.InvalidGitRepositoryError as exception:
        raise ValueError("repo not found") from exception


def run_test_command(test_command: str) -> bool:
    command = test_command.split(" ")
    logger.info(f"Running command {command}")
    completed_proccess = subprocess.run(command)
    command_successful = completed_proccess.returncode == 0
    return command_successful


def fetch_tags(repo: Repo) -> None:
    logger.info("Fetching Tags")
    for remote in repo.remotes:
        fetch_info = remote.fetch(tags=True)
        for info in fetch_info:
            logger.info(f"Successfully fetched {info.ref} at {info.commit}")


def get_latest_version(repo: Repo) -> str:
    logger.info("Get latest Tag")
    latest_tag: str = repo.git.describe(abbrev=0, tags=True)
    return latest_tag


def chekcout_paths_from_reference(
    repo: Repo, reference: str, paths: Iterable[Path]
) -> None:
    """Using checkout in sparse mode to bring only one path from reference"""
    for path in paths:
        logger.info("Get {path} from {reference}")
        repo.git.checkout(reference, "--", path)


def discard_changes(repo: Repo) -> None:
    logger.info("Discard Changes")
    repo.git.reset("HEAD")
    repo.git.checkout(".")
